/*
*   The LoadWeather class loads the weather from the Environment Canada server at https://dd.weather.gc.ca/marine_weather/xml/pacific/ 
*   A small php proxy is used to load the weather via XMLHttpRequest, CORS prevents loading from the Environment Canada server directly.
*   localStorage is used to save and retrieve user selected forecast areas.
*/
'use strict';
function LoadWeather(wxdiv){

    //Div that holds the formatted weather
    const wxDiv = document.getElementById(wxdiv);

    //Forecast area files available from Environment Canada server
    const forecastAreas = new Map([['jdf','m0000009_e.xml'],['wcvis','m0000065_e.xml'],['wcvin','m0000043_e.xml'],['cc','m0000140_e.xml'],
                                  ['qcs','m0000112_e.xml'],['jhn','m0000010_e.xml'],['sog','m0000028_e.xml'],['hro','m0000064_e.xml']]);

    //Forecasts grouped by area, and set to true if user has selected forecast area checkbox

    let selectedAreas = [{'name': 'westisland' , 'selected': true, 'areas' :['jdf','wcvis','wcvin']},
                           {'name': 'northisland', 'selected': true, 'areas' :['wcvin','cc','qcs','jhn']},
                           {'name': 'eastisland' , 'selected': true, 'areas' :['sog']},
                           {'name': 'gulfisland' , 'selected': true, 'areas' :['sog','hro']},
                           {'name': 'southisland', 'selected': true, 'areas' :['jdf','hro']}];

    //Set that holds all combined forecast areas
    let forecastSet = new Set();

    /*
    *   Run on page load to retrieve any selectedAreas saved previously from localStorage, and fetch the saved weather areas.
    */
    this.start = function(){
        for(let area in selectedAreas){
            let isselected = window.localStorage.getItem(selectedAreas[area].name);
			let ckbx = document.getElementById(selectedAreas[area].name);

            if(isselected){
                selectedAreas[area].selected = isselected;

                //Set checkbox to saved value
                ckbx.checked = isselected;
            }else{

                //Either true or no data was available, select area by default
                window.localStorage.setItem(selectedAreas[area].name,false);
                selectedAreas[area].selected = false;
                ckbx.checked = false;
            }
        }
        this.fetchWeather();
    }

    /*
    *   Run on checkbox change to save selectedAreas to localStorage, and fetch the selected weather areas.
    */
    this.saveAreas = function(evt){

        for(let area in selectedAreas){

            //Confirm that checkbox name matches one of the selected areas
            if(evt.currentTarget.name == selectedAreas[area].name)
				selectedAreas[area].selected = evt.currentTarget.checked;
                window.localStorage.setItem(selectedAreas[area].name,evt.currentTarget.checked);
        }
        this.fetchWeather();
    }

    /*
    * Fetchs the weather from the Environment Canada website via the php proxy.
    */
    this.fetchWeather = function(){

        //holds the fetched forecast
        let forecast = {};
        wxDiv.innerHTML = "";

        //Fill forecastSet with area filenames
        forecastSet.clear();
        for(let area in selectedAreas){
			if(selectedAreas[area].selected){
            	for(let subarea in selectedAreas[area].areas){
                	forecastSet.add(forecastAreas.get(selectedAreas[area].areas[subarea]));
            	}
			}
        }

        function fetcher(v1,v2,set){
			let xf = new XMLHttpRequest();
			xf.responseType = 'document';
			xf.addEventListener('load',weatherSuccess);
			xf.addEventListener('error',weatherFail);
			xf.addEventListener('abort',weatherFail);
			xf.open('GET','p.php?wx=' + v1);
			xf.send();
        }

		//Loop through each forecast area and fetch the forecast
        forecastSet.forEach(fetcher);

        function weatherFail(evt){
            wxDiv.innerHTML += "<br>Error fetching wx<br>";
        }

		//Display the selected forecast areas in the forecast div
        function weatherSuccess(evt){
			let wxdoc = evt.target.responseXML;
			let fcst = wxdoc.getElementsByTagName('regularforecast');
			let text = "";

			for(let i =0;i<fcst.length;i++){
				let infcst = fcst[i].childNodes;
				for(let j = 0;j<infcst.length;j++){
					let wxdetails = infcst[j].childNodes;
					for(let k = 0;k < wxdetails.length;k++){
						if(wxdetails[k].tagName != undefined && wxdetails[k].outerHTML != undefined)
							text += wxdetails[k].tagName + ": " + wxdetails[k].outerHTML + "<br>";
					}
				}
			}

            wxDiv.innerHTML += text + "<br><br>";
		}
    }
}
