/*
*	The MapPopup class shows a pop-up of the marina's current location on the
*	web page using the leaflet library to load and display the map.
*	This class is used for both 'Unit 5: Writing Javascript' and 'Unit 6: Using Libraries' of the Comp 266 course.
*	
*	The class requires the mappopup.css stylesheet to properly display the map div.
*/

function MapPopup(){
	
	// Create the div used to hold the map, add the hidden style,
	// and append to the html body.
	const popUpDiv = document.createElement('div');
	popUpDiv.id = 'popupdiv';
	popUpDiv.classList.add('hidden');
	document.body.appendChild(popUpDiv);
	
	// Create the leaflet map.
	let map = L.map('popupdiv',{
		zoom: 13,
		zoomControl: false
	});
	
	// Set the map to use openstreetmap as the data source.
	L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
    attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
	}).addTo(map);
	
	// Create marina marker and add it to the map.
	let marker = L.marker();
	marker.setLatLng([48,123]);//Have to set a default position or the marker throws an error
	marker.addTo(map);

	/*
	*	The start function is called to load all the <a> tags containing the data-lat property.
	*	The <a> tags then have the popIn function bound to the mouseover event, and the
	*	popOut function bound to the mouseout event.
	*/
	this.start = function(){
		
		const atags = document.querySelectorAll('a[data-lat]');
		
		for(let a of atags){
			a.addEventListener('mouseover',this.popIn);
			a.addEventListener('mouseout',this.popOut);
		}
	}
	
	/*
	*	The popIn function sets the mapdiv position based on the mouse position, then
	*	loads the lat and lon from the <a> tag that triggered the event.  The map and
	*	marker are both set to the lat lon position, the marker popup is set to the 
	*	<a> title, and the map is displayed.
	*/
	this.popIn = function(event){
		
		// Map div offset position.
		let pxtop = event.screenY - 310;
		let pxleft = event.screenX + 20;
		
		// OpenStreetMap centre position.
		let panlat = event.target.dataset.lat;
		let panlon = event.target.dataset.lon;
		let latlng = L.latLng(panlat, panlon);
		
		popUpDiv.style.top = pxtop + 'px';
		popUpDiv.style.left = pxleft + 'px';
		popUpDiv.classList.remove('hidden');
		
		marker.setLatLng(latlng);
		marker.bindPopup(event.target.title);
		marker.openPopup();
		
		map.panTo(latlng);
		map.invalidateSize(); // Have to call invalidateSize if the map div was previously hidden.
	}
	
	/*
	*	The popOut function simply adds the 'hidden'
	*	class to the map div to hide the map.
	*/
	this.popOut = function(event){
		popUpDiv.classList.add('hidden');
	}
}

// Create and start the MapPopup class
let mpopup = new MapPopup();
mpopup.start();